
#include "mex.h"
#include "matrix.h"

#include "ComplexSF.cpp"

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
 	
    if(nrhs< 3 || nrhs > 6) {
	printf("\nUsage: [up low] = ComplexFilter(f ,coeffcients, order = 1, type = 0, boarder = 0, real = 0)\n\n",nrhs);
	printf(" Applys complex filter. \n\n");
	printf(" Parameters:\n");
	printf("   f - 2D input image of type double \n");
	printf("   coefficients - square matrix containing the filter coeffcients.\n" );
	printf("   order - approximation order, possible values 1 or 2\n" );
	printf("   type - possible values 0,1 for central or forward/backward differences\n");
	printf("   boarder - defines width of the boundary where the derivative is NOT computed\n" );
	printf("   real - possible values 0,1. For real=1, computes the filter as if the lower triangle\n" );
	printf("          of the coeffcient matrix is zero.  \n\n" );
	printf(" Return value [up low] contains the steerable filter with pos. and neg rot. behavior.\n\n");
    	return;
	} else if(nlhs>2) {
	printf("Too many output arguments\n");
    return;
	}

    int pcnt = 0;
    const mxArray *Img;
    Img = prhs[pcnt++];       
    const int numdim = mxGetNumberOfDimensions(Img);
    const int *dims = mxGetDimensions(Img);
    const int d1 = dims[0];
    const int d2 = dims[1];

    double *realimg = mxGetPr(Img);
    double *imagimg = mxGetPi(Img);

    const mxArray *alpha = prhs[pcnt++];
    double *alphareal = mxGetPr(alpha);
    double *alphaimag = mxGetPi(alpha);
    
    int degree = mxGetN(alpha);

    int boarder = 0;
    int order = 1;
    int central = 0;
    int real = 0;

    if (nrhs > 3)
	{
	double *f_order = mxGetPr(prhs[pcnt++]);
	if (*f_order > 1.5)
		order = 2;
	}
    if (nrhs > 4)
	{
	double *f_central = mxGetPr(prhs[pcnt++]);
	central = (int)(*f_central);
	}
    if (nrhs > 5)
	{
	double *f_boarder = mxGetPr(prhs[pcnt++]);
	boarder = (int) *f_boarder; 
	}
    if (nrhs > 6)
	{
	double *f_real = mxGetPr(prhs[pcnt++]);
	real = (int) *f_real; 
	}



    int dsz[3]; dsz[0] = d1, dsz[1] = d2, dsz[2] = degree;

    plhs[0] = mxCreateNumericArray(3,dsz,mxDOUBLE_CLASS,mxCOMPLEX);
    double *resrealUp = mxGetPr(plhs[0]);
    double *resimagUp = mxGetPi(plhs[0]);

    if (real == 0)
	{
	plhs[1] = mxCreateNumericArray(3,dsz,mxDOUBLE_CLASS,mxCOMPLEX);
	double *resrealLow = mxGetPr(plhs[1]);
	double *resimagLow = mxGetPi(plhs[1]);
        ComplexSF::Genericfilter(realimg,imagimg,resrealLow,resimagLow,resrealUp,resimagUp,
				alphareal,alphaimag,
			       boarder,degree,central,order,d1,d2);

	}
    else
	{
        ComplexSF::GenericfilterReal(realimg,imagimg,resrealUp,resimagUp,
				alphareal,alphaimag,
			       boarder,degree,central,order,d1,d2);
	}



}

