
#include "mex.h"
#include "matrix.h"

#include "ComplexSF.cpp"

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
 	
    if(nrhs< 3 || nrhs > 6) {
	printf("\nUsage: up = ComplexFilterHolo(f , coeffcients, order = 1,type = 0, boarder = 0\n\n",nrhs);	
	printf(" Applys complex filter. Only z-derivatives are considered. Filter kernel is a \n");
	printf(" holomorphic function windowed by the rotation symmetric basis kernel.\n\n");
	printf(" Parameters:\n");
	printf("   f - 2D input image of type double \n");
	printf("   coefficients - row vector (z-derivatives) or column vector (zbar-derivatives)\n");
	printf("       containing the filter coeffcients\n");
	printf("   order - approximation order, possible values 1 or 2\n" );
	printf("   type - possible values 0,1 for central or forward/backward differences\n");
	printf("   boarder - defines width of the boundary where the derivative is NOT computed\n\n" );
	printf(" Return value up contains the result, same size as f.\n\n");
    	return;
	} else if(nlhs>2) {
	printf("Too many output arguments\n");
    return;
	}

    int pcnt = 0;
    const mxArray *Img;
    Img = prhs[pcnt++];       
    const int numdim = mxGetNumberOfDimensions(Img);
    const int *dims = mxGetDimensions(Img);
    const int d1 = dims[0];
    const int d2 = dims[1];

    double *realimg = mxGetPr(Img);
    double *imagimg = mxGetPi(Img);

    const mxArray *alpha = prhs[pcnt++];
    const int a1 = mxGetM(alpha);
    const int a2 = mxGetN(alpha);
    double *alphareal = mxGetPr(alpha);
    double *alphaimag = mxGetPi(alpha);
    
    int degree = 0;
    int sign = 1;
    if (a1 == 1)
    {
	degree = a2;
        sign = 1;
    }
    else 
    {
	degree = a1;
	sign = -1;
    }

    int boarder = 0;
    int order = 1;
    int central = 0;

    if (nrhs > 2)
	{
	double *f_order = mxGetPr(prhs[pcnt++]);
	if (*f_order > 1.5)
		order = 2;
	}
    if (nrhs > 3)
	{
	double *f_central = mxGetPr(prhs[pcnt++]);
	central = (int)(*f_central);
	}
    if (nrhs > 4)
	{
	double *f_boarder = mxGetPr(prhs[pcnt++]);
	boarder = (int) *f_boarder; 

	}


    int dsz[3]; dsz[0] = d1, dsz[1] = d2, dsz[2] = degree;

    plhs[0] = mxCreateNumericArray(3,dsz,mxDOUBLE_CLASS,mxCOMPLEX);
    double *resreal = mxGetPr(plhs[0]);
    double *resimag = mxGetPi(plhs[0]);


    ComplexSF::Genericfilter1(realimg,imagimg,resreal,resimag,
				alphareal,alphaimag, sign,
			       boarder,degree,central,order,d1,d2);



}

