
#include <math.h>
#include <string.h>
#include <stdlib.h>

template <class T> inline void swap(T& x, T& y) { T t=x; x=y; y=t; }


class ComplexSF
{
	public:

	// generic differneces
	static void partialDeriv(double *realimg, double *imagimg, double *resreal, double *resimag, 
			  const int boarder, const int sign, const int central, const int order,
			  const int d1, const int d2);


	// central difference
        static void partialDerivCentral(double *realsrc, double *imagsrc, 
			  double *realdest, double *imagdest, 
			  const int boarder, const int conj, const int m, const int n);
        static void partialDerivCentral2(double *realsrc, double *imagsrc, 
			  double *realdest, double *imagdest, 
			  const  int boarder, const int conj, const int m, const int n);

	// forward/backward difference
        static void partialDerivForward(double *realsrc, double *imagsrc, 
			  double *realdest, double *imagdest, 
			  const int boarder, const int conj, const int m, const int n);
        static void partialDerivBackward(double *realsrc, double *imagsrc, 
			  double *realdest, double *imagdest, 
			  const int boarder, const int conj, const int m, const int n);
        static void partialDerivForward2(double *realsrc, double *imagsrc, 
			  double *realdest, double *imagdest, 
			  const int boarder, const int conj, const int m, const int n);
        static void partialDerivBackward2(double *realsrc, double *imagsrc, 
			  double *realdest, double *imagdest, 
			  const int boarder, const int conj, const int m, const int n);


	// Filter routine
	static void Genericfilter(double *realsrc, double *imagsrc, 
			        double *realdestLow, double *imagdestLow, 
				double *realdestUp, double *imagdestUp, 
				double *realcoeffs, double *imagcoeffs,
				const int boarder, const int degree,
				const int central, const int order, 
				const int m, const int n);

	// Real Filter
	static void GenericfilterReal(double *realsrc, double *imagsrc, 
				double *realdestUp, double *imagdestUp, 
				double *realcoeffs, double *imagcoeffs,
				const int boarder, const int degree,
				const int central, const int order, 
				const int m, const int n);

	// complex analytic
	static void Genericfilter1(double *realsrc, double *imagsrc, 
			        double *realdest, double *imagdest, 
				double *realcoeffs, double *imagcoeffs,
				const int sign,
				const int boarder, const int degree,
				const int central, const int order, 
				const int m, const int n);



};


void ComplexSF::partialDeriv(double *realimg, double *imagimg, double *resreal, double *resimag, 
			  const int boarder, const int sign, const int central, const int order,
			  const int d1, const int d2)
{
	
	if (order == 1)
	{
		if (central == 0)        
			ComplexSF::partialDerivCentral(realimg,imagimg,resreal,resimag,boarder,sign,d1,d2);
		else if (central == 1)        
			ComplexSF::partialDerivForward(realimg,imagimg,resreal,resimag,boarder,sign,d1,d2);
		else if (central == -1)        
			ComplexSF::partialDerivBackward(realimg,imagimg,resreal,resimag,boarder,sign,d1,d2);
	}
	else
	{
		if (central == 0)        
			ComplexSF::partialDerivCentral2(realimg,imagimg,resreal,resimag,boarder,sign,d1,d2);
		else if (central == 1)        
			ComplexSF::partialDerivForward2(realimg,imagimg,resreal,resimag,boarder,sign,d1,d2);
		else if (central == -1)        
			ComplexSF::partialDerivBackward2(realimg,imagimg,resreal,resimag,boarder,sign,d1,d2);
	}

}




void ComplexSF::partialDerivCentral2(double *realsrc, double *imagsrc, double *realdest, double *imagdest, const int boarder, const int conj, const int m, const int n)
{	
	if (imagsrc != 0)
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jm = j*m;
			for (int k = boarder; k < m-boarder ; k++)
			{
				int jmk = jm+k;
				int ddj = ((j+2)%n)*m + k;
				int dj = ((j+1)%n)*m + k;
				int _dj = ((n+j-1)%n)*m + k;
				int _ddj = ((n+j-2)%n)*m + k;
				int ddk = jm + (k+2)%m;
				int dk = jm + (k+1)%m;
				int _dk = jm + (m+k-1)%m;
				int _ddk = jm + (m+k-2)%m;
				realdest[jmk] = ((-0.2*realsrc[ddj] + realsrc[dj] - realsrc[_dj] + realsrc[_ddj]*0.2)
			 	      -    conj*(-0.2*imagsrc[ddk] + imagsrc[dk] - imagsrc[_dk] + imagsrc[_ddk]*0.2))*1.5;
				imagdest[jmk] = ((-0.2*imagsrc[ddj] + imagsrc[dj] - imagsrc[_dj] + imagsrc[_ddj]*0.2)
					 + conj*(-0.2*realsrc[ddk] + realsrc[dk] - realsrc[_dk] + realsrc[_ddk]*0.2))*1.5;
			}
		}
	}
	else
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jm = j*m;
			for (int k = boarder; k < m-boarder ; k++)
			{
				int jmk = jm+k;
				int ddj = ((j+2)%n)*m + k;
				int dj = ((j+1)%n)*m + k;
				int _dj = ((n+j-1)%n)*m + k;
				int _ddj = ((n+j-2)%n)*m + k;
				int ddk = jm + (k+2)%m;
				int dk = jm + (k+1)%m;
				int _dk = jm + (m+k-1)%m;
				int _ddk = jm + (m+k-2)%m;
				realdest[jmk] = (-0.2*realsrc[ddj] + realsrc[dj] - realsrc[_dj] + realsrc[_ddj]*0.2)*1.5;
				imagdest[jmk] = conj*(-0.2*realsrc[ddk] + realsrc[dk] - realsrc[_dk] + realsrc[_ddk]*0.2)*1.5;
			}
		}
	}
}



void ComplexSF::partialDerivCentral(double *realsrc, double *imagsrc, double *realdest, double *imagdest, const int boarder, const int conj, const int m, const int n)
{	
	if (imagsrc != 0)
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jm = j*m;
			int dj0 = ((j+1)%n)*m;
			int _dj0 = ((n+j-1)%n)*m;
			for (int k = boarder; k < m-boarder ; k++)
			{
				int jmk = jm+k;
				int dj  =  dj0 + k;
				int _dj = _dj0 + k;
				int dk = jm + (k+1)%m;
				int _dk = jm + (m+k-1)%m;
				realdest[jmk] = (realsrc[dj] - realsrc[_dj]) -  conj*(imagsrc[dk] - imagsrc[_dk]);
				imagdest[jmk] = (imagsrc[dj] - imagsrc[_dj]) +  conj*(realsrc[dk] - realsrc[_dk]);
			}
		}
	}
	else
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jm = j*m;
			int dj0 = ((j+1)%n)*m;
			int _dj0 = ((n+j-1)%n)*m;
			for (int k = boarder; k < m-boarder ; k++)
			{
				int jmk = jm+k;
				int dj  =  dj0 + k;
				int _dj = _dj0 + k;
				int dk = jm + (k+1)%m;
				int _dk = jm + (m+k-1)%m;
				realdest[jmk] = (realsrc[dj] - realsrc[_dj]);
				imagdest[jmk] = conj*(realsrc[dk] - realsrc[_dk]);
			}
		}
	}
}




void ComplexSF::partialDerivForward(double *realsrc, double *imagsrc, double *realdest, double *imagdest, const int boarder, const int conj, const int m, const int n)
{	
	if (imagsrc != 0)
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jm = j*m;
			int jmp1 = ((j+1)%n)*m;
			for (int k = boarder; k < m-boarder ; k++)
			{
				int jm00 = jm   + k;
				int jm10 = jm   + (k+1)%m;
				int jm01 = jmp1 + k;				
				int jm11 = jmp1 + (k+1)%m;

				realdest[jm00] =  (       (realsrc[jm11] + realsrc[jm01] - realsrc[jm10] - realsrc[jm00]) 
						-  conj*(imagsrc[jm11] + imagsrc[jm10] - imagsrc[jm01] - imagsrc[jm00]) );
				imagdest[jm00] =  (       (imagsrc[jm11] + imagsrc[jm01] - imagsrc[jm10] - imagsrc[jm00])      
						+  conj*(realsrc[jm11] + realsrc[jm10] - realsrc[jm01] - realsrc[jm00]) );

			}
		}
	}
	else
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jm = j*m;
			int jmp1 = ((j+1)%n)*m;
			for (int k = boarder; k < m-boarder ; k++)
			{
				int jm00 = jm   + k;
				int jm10 = jm   + (k+1)%m;
				int jm01 = jmp1 + k;				
				int jm11 = jmp1 + (k+1)%m;

				realdest[jm00] = (realsrc[jm11] + realsrc[jm01] - realsrc[jm10] - realsrc[jm00]);
				imagdest[jm00] = conj*(realsrc[jm11] + realsrc[jm10] - realsrc[jm01] - realsrc[jm00]);

			}
		}
	}
}


void ComplexSF::partialDerivBackward(double *realsrc, double *imagsrc, double *realdest, double *imagdest, const int boarder, const int conj, const int m, const int n)
{	
	if (imagsrc != 0)
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jm = j*m;
			int jmp1 = ((j-1+n)%n)*m;
			for (int k = boarder; k < m-boarder ; k++)
			{
				int jm00 = jm   + k;
				int jm10 = jm   + (k-1+m)%m;
				int jm01 = jmp1 + k;				
				int jm11 = jmp1 + (k-1+m)%m;

				realdest[jm00] =  (-(realsrc[jm11] + realsrc[jm01] - realsrc[jm10] -realsrc[jm00])
					+  conj*(imagsrc[jm11] + imagsrc[jm10] - imagsrc[jm01] - imagsrc[jm00]) );
				imagdest[jm00] =  (-(imagsrc[jm11] + imagsrc[jm01] - imagsrc[jm10] - imagsrc[jm00])    
					-  conj*(realsrc[jm11] + realsrc[jm10] - realsrc[jm01] - realsrc[jm00]) );

			}
		}
	}
	else
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jm = j*m;
			int jmp1 = ((j-1+n)%n)*m;
			for (int k = boarder; k < m-boarder ; k++)
			{
				int jm00 = jm   + k;
				int jm10 = jm   + (k-1+m)%m;
				int jm01 = jmp1 + k;				
				int jm11 = jmp1 + (k-1+m)%m;

				realdest[jm00] = -(realsrc[jm11] + realsrc[jm01] - realsrc[jm10] - realsrc[jm00]);
				imagdest[jm00] = -conj*(realsrc[jm11] + realsrc[jm10] - realsrc[jm01] - realsrc[jm00]);

			}
		}
	}
}


#define A (0.013020833333333*4.0)
#define B (0.070312499999998*4.0)
#define C (0.023437500000000*4.0)
#define D (0.351562500000001*4.0)
#define E (0.023437500000000*4.0)



void ComplexSF::partialDerivForward2(double *realsrc, double *imagsrc, double *realdest, double *imagdest, const int boarder, const int conj, const int m, const int n)
{	
	if (imagsrc != 0)
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jmm1 = ((j-1+n)%n)*m;
			int jm = j*m;
			int jmp1 = ((j+1)%n)*m;
			int jmp2 = ((j+2)%n)*m;
			for (int k = boarder; k < m-boarder ; k++)
			{

				int jmaa = jmm1   + (k-1+m)%m;
				int jm0a = jmm1   + k;
				int jm1a = jmm1   + (k+1)%m;
				int jm2a = jmm1   + (k+2)%m;

				int jma0 = jm   + (k-1+m)%m;
				int jm00 = jm   + k;
				int jm10 = jm   + (k+1)%m;
				int jm20 = jm   + (k+2)%m;

				int jma1 = jmp1   + (k-1+m)%m;
				int jm01 = jmp1   + k;
				int jm11 = jmp1   + (k+1)%m;
				int jm21 = jmp1   + (k+2)%m;

				int jma2 = jmp2   + (k-1+m)%m;
				int jm02 = jmp2   + k;
				int jm12 = jmp2   + (k+1)%m;
				int jm22 = jmp2   + (k+2)%m;

			realdest[jm00] = (-A*realsrc[jmaa] + B*realsrc[jma0] - B*realsrc[jma1] + A*realsrc[jma2] +
					  C*realsrc[jm0a] - D*realsrc[jm00] + D*realsrc[jm01] - C*realsrc[jm02] +
					  C*realsrc[jm1a] - D*realsrc[jm10] + D*realsrc[jm11] - C*realsrc[jm12]  
					  -A*realsrc[jm2a] + B*realsrc[jm20] - B*realsrc[jm21] + A*realsrc[jm22] ) 
					+ conj *
					( A*imagsrc[jmaa] - E*imagsrc[jma0] - E*imagsrc[jma1] + A*imagsrc[jma2]
					  -B*imagsrc[jm0a] + D*imagsrc[jm00] + D*imagsrc[jm01] - B*imagsrc[jm02] +
					   B*imagsrc[jm1a] - D*imagsrc[jm10] - D*imagsrc[jm11] + B*imagsrc[jm12]  
					  -A*imagsrc[jm2a] + E*imagsrc[jm20] + E*imagsrc[jm21] - A*imagsrc[jm22] ); 

			imagdest[jm00] = (-A*imagsrc[jmaa] + B*imagsrc[jma0] - B*imagsrc[jma1] + A*imagsrc[jma2] +
					  C*imagsrc[jm0a] - D*imagsrc[jm00] + D*imagsrc[jm01] - C*imagsrc[jm02] +
					  C*imagsrc[jm1a] - D*imagsrc[jm10] + D*imagsrc[jm11] - C*imagsrc[jm12]  
					  -A*imagsrc[jm2a] + B*imagsrc[jm20] - B*imagsrc[jm21] + A*imagsrc[jm22] ) 
					- conj *
					( A*realsrc[jmaa] - E*realsrc[jma0] - E*realsrc[jma1] + A*realsrc[jma2] +
					  -B*realsrc[jm0a] + D*realsrc[jm00] + D*realsrc[jm01] - B*realsrc[jm02] +
					   B*realsrc[jm1a] - D*realsrc[jm10] - D*realsrc[jm11] + B*realsrc[jm12]  
					  -A*realsrc[jm2a] + E*realsrc[jm20] + E*realsrc[jm21] - A*realsrc[jm22] ); 



			}
		}
	}
	else
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jmm1 = ((j-1+n)%n)*m;
			int jm = j*m;
			int jmp1 = ((j+1)%n)*m;
			int jmp2 = ((j+2)%n)*m;
			for (int k = boarder; k < m-boarder ; k++)
			{
				int jmaa = jmm1   + (k-1+m)%m;
				int jm0a = jmm1   + k;
				int jm1a = jmm1   + (k+1)%m;
				int jm2a = jmm1   + (k+2)%m;

				int jma0 = jm   + (k-1+m)%m;
				int jm00 = jm   + k;
				int jm10 = jm   + (k+1)%m;
				int jm20 = jm   + (k+2)%m;

				int jma1 = jmp1   + (k-1+m)%m;
				int jm01 = jmp1   + k;
				int jm11 = jmp1   + (k+1)%m;
				int jm21 = jmp1   + (k+2)%m;

				int jma2 = jmp2   + (k-1+m)%m;
				int jm02 = jmp2   + k;
				int jm12 = jmp2   + (k+1)%m;
				int jm22 = jmp2   + (k+2)%m;

			realdest[jm00] = (-A*realsrc[jmaa] + B*realsrc[jma0] - B*realsrc[jma1] + A*realsrc[jma2] +
					  C*realsrc[jm0a] - D*realsrc[jm00] + D*realsrc[jm01] - C*realsrc[jm02] +
					  C*realsrc[jm1a] - D*realsrc[jm10] + D*realsrc[jm11] - C*realsrc[jm12]  
					  -A*realsrc[jm2a] + B*realsrc[jm20] - B*realsrc[jm21] + A*realsrc[jm22] );
					

			imagdest[jm00] = - conj *
					( A*realsrc[jmaa] - E*realsrc[jma0] - E*realsrc[jma1] + A*realsrc[jma2] +
					  -B*realsrc[jm0a] + D*realsrc[jm00] + D*realsrc[jm01] - B*realsrc[jm02] +
					   B*realsrc[jm1a] - D*realsrc[jm10] - D*realsrc[jm11] + B*realsrc[jm12]  
					  -A*realsrc[jm2a] + E*realsrc[jm20] + E*realsrc[jm21] - A*realsrc[jm22] ); 



			}
		}
	}
}



void ComplexSF::partialDerivBackward2(double *realsrc, double *imagsrc, double *realdest, double *imagdest, const int boarder, const int conj, const int m, const int n)
{	
	if (imagsrc != 0)
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jmm1 = ((j-2+n)%n)*m;
			int jm =   ((j-1+n)%n)*m;
			int jmp1 = j*m;
			int jmp2 = ((j+1)%n)*m;
			for (int k = boarder; k < m-boarder ; k++)
			{

				int jmaa = jmm1   + (k-2+m)%m;
				int jm0a = jmm1   + (k-1+m)%m;
				int jm1a = jmm1   + k;
				int jm2a = jmm1   + (k+1)%m;

				int jma0 = jm   + (k-2+m)%m;
				int jm00 = jm   + (k-1+m)%m;
				int jm10 = jm   + k;
				int jm20 = jm   + (k+1)%m;

				int jma1 = jmp1   + (k-2+m)%m;
				int jm01 = jmp1   + (k-1+m)%m;
				int jm11 = jmp1   + k;
				int jm21 = jmp1   + (k+1)%m;

				int jma2 = jmp2   + (k-2+m)%m;
				int jm02 = jmp2   + (k-1+m)%m;
				int jm12 = jmp2   + k;
				int jm22 = jmp2   + (k+1)%m;

			realdest[jm11] = (-A*realsrc[jmaa] + B*realsrc[jma0] - B*realsrc[jma1] + A*realsrc[jma2] +
					  C*realsrc[jm0a] - D*realsrc[jm00] + D*realsrc[jm01] - C*realsrc[jm02] +
					  C*realsrc[jm1a] - D*realsrc[jm10] + D*realsrc[jm11] - C*realsrc[jm12]  
					  -A*realsrc[jm2a] + B*realsrc[jm20] - B*realsrc[jm21] + A*realsrc[jm22] ) 
					+ conj *
					( A*imagsrc[jmaa] - E*imagsrc[jma0] - E*imagsrc[jma1] + A*imagsrc[jma2]
					  -B*imagsrc[jm0a] + D*imagsrc[jm00] + D*imagsrc[jm01] - B*imagsrc[jm02] +
					   B*imagsrc[jm1a] - D*imagsrc[jm10] - D*imagsrc[jm11] + B*imagsrc[jm12]  
					  -A*imagsrc[jm2a] + E*imagsrc[jm20] + E*imagsrc[jm21] - A*imagsrc[jm22] ); 

			imagdest[jm11] = (-A*imagsrc[jmaa] + B*imagsrc[jma0] - B*imagsrc[jma1] + A*imagsrc[jma2] +
					  C*imagsrc[jm0a] - D*imagsrc[jm00] + D*imagsrc[jm01] - C*imagsrc[jm02] +
					  C*imagsrc[jm1a] - D*imagsrc[jm10] + D*imagsrc[jm11] - C*imagsrc[jm12]  
					  -A*imagsrc[jm2a] + B*imagsrc[jm20] - B*imagsrc[jm21] + A*imagsrc[jm22] ) 
					- conj *
					( A*realsrc[jmaa] - E*realsrc[jma0] - E*realsrc[jma1] + A*realsrc[jma2] +
					  -B*realsrc[jm0a] + D*realsrc[jm00] + D*realsrc[jm01] - B*realsrc[jm02] +
					   B*realsrc[jm1a] - D*realsrc[jm10] - D*realsrc[jm11] + B*realsrc[jm12]  
					  -A*realsrc[jm2a] + E*realsrc[jm20] + E*realsrc[jm21] - A*realsrc[jm22] ); 



			}
		}
	}
	else
	{
		for (int j = boarder; j < n-boarder ; j++)
		{
			int jmm1 = ((j-1+n)%n)*m;
			int jm = j*m;
			int jmp1 = ((j+1)%n)*m;
			int jmp2 = ((j+2)%n)*m;
			for (int k = boarder; k < m-boarder ; k++)
			{
				int jmaa = jmm1   + (k-2+m)%m;
				int jm0a = jmm1   + (k-1+m)%m;
				int jm1a = jmm1   + k;
				int jm2a = jmm1   + (k+1)%m;

				int jma0 = jm   + (k-2+m)%m;
				int jm00 = jm   + (k-1+m)%m;
				int jm10 = jm   + k;
				int jm20 = jm   + (k+1)%m;

				int jma1 = jmp1   + (k-2+m)%m;
				int jm01 = jmp1   + (k-1+m)%m;
				int jm11 = jmp1   + k;
				int jm21 = jmp1   + (k+1)%m;

				int jma2 = jmp2   + (k-2+m)%m;
				int jm02 = jmp2   + (k-1+m)%m;
				int jm12 = jmp2   + k;
				int jm22 = jmp2   + (k+1)%m;

			realdest[jm11] = (-A*realsrc[jmaa] + B*realsrc[jma0] - B*realsrc[jma1] + A*realsrc[jma2] +
					  C*realsrc[jm0a] - D*realsrc[jm00] + D*realsrc[jm01] - C*realsrc[jm02] +
					  C*realsrc[jm1a] - D*realsrc[jm10] + D*realsrc[jm11] - C*realsrc[jm12]  
					  -A*realsrc[jm2a] + B*realsrc[jm20] - B*realsrc[jm21] + A*realsrc[jm22] );
					

			imagdest[jm11] = - conj *
					( A*realsrc[jmaa] - E*realsrc[jma0] - E*realsrc[jma1] + A*realsrc[jma2] +
					  -B*realsrc[jm0a] + D*realsrc[jm00] + D*realsrc[jm01] - B*realsrc[jm02] +
					   B*realsrc[jm1a] - D*realsrc[jm10] - D*realsrc[jm11] + B*realsrc[jm12]  
					  -A*realsrc[jm2a] + E*realsrc[jm20] + E*realsrc[jm21] - A*realsrc[jm22] ); 



			}
		}
	}
}

void ComplexSF::Genericfilter(double *realsrc, double *imagsrc, 
			        double *realdestLow, double *imagdestLow, 
				double *realdestUp, double *imagdestUp, 
				double *realcoeffs, double *imagcoeffs,
				const int boarder, const int degree,
				const int central, const int order, 
				const int m, const int n)
{

	int mn = m*n;
	double *reG = (double*) malloc(sizeof(double)*mn);
	double *imG = (double*) malloc(sizeof(double)*mn);
	double *reG_ = (double*) malloc(sizeof(double)*mn);
	double *imG_ = (double*) malloc(sizeof(double)*mn);
	double *reGt = (double*) malloc(sizeof(double)*mn);
	double *imGt = (double*) malloc(sizeof(double)*mn);
	double *reGt_ = (double*) malloc(sizeof(double)*mn);
	double *imGt_ = (double*) malloc(sizeof(double)*mn);


	memcpy(reGt,realsrc,sizeof(double)*mn);
	if (imagsrc != 0)
		memcpy(imGt,imagsrc,sizeof(double)*mn);
	else
	{
		for (int i = 0;i < mn;i++) imGt[i] = 0.0;
	}		

	for (int k = 0;k < degree;k++)
	{
		if (k > 0)
		{
			if (k%2 == 0)	
				partialDeriv(reGt,imGt,reGt_,imGt_,boarder,1,-central,order,m,n);
			else
				partialDeriv(reGt,imGt,reGt_,imGt_,boarder,1,central,order,m,n);
			swap(reGt,reGt_);
			swap(imGt,imGt_);
		}

		memcpy(reG,reGt,sizeof(double)*mn);
		memcpy(imG,imGt,sizeof(double)*mn);
		double *redest;
		double *imdest;


		for (int j = 0;j < degree;j++)
		{
			if (j > 0)
			{
				if ((k+j)%2 == 0)
					partialDeriv(reG,imG,reG_,imG_,boarder,-1,-central,order,m,n);
				else
					partialDeriv(reG,imG,reG_,imG_,boarder,-1,central,order,m,n);
				swap(reG,reG_);
				swap(imG,imG_);
			}

			if (imagcoeffs == 0)
			{
		
				double realcoeff = realcoeffs[k+j*degree];
	
				if (k-j >= 0)
				{
					redest = realdestUp + mn*(k-j);
					imdest = imagdestUp + mn*(k-j);
				}
				else
				{
					redest = realdestLow + mn*(j-k);
					imdest = imagdestLow + mn*(j-k);
				}

				for (int i = 0;i < mn; i++)
				{
					redest[i] += realcoeff*reG[i];
					imdest[i] += realcoeff*imG[i];
				}

			}
			else
			{
		
				double realcoeff = realcoeffs[k*degree+j];
				double imagcoeff = imagcoeffs[k*degree+j];
				if (k-j >= 0)
				{
					redest = realdestUp + mn*(k-j);
					imdest = imagdestUp + mn*(k-j);
				}
				else
				{
					redest = realdestLow + mn*(j-k);
					imdest = imagdestLow + mn*(j-k);
				}

				for (int i = 0;i < mn; i++)
				{
					redest[i] += realcoeff*reG[i] - imagcoeff*imG[i];
					imdest[i] += realcoeff*imG[i] + imagcoeff*reG[i];
				}

			}

		}
	}

	free(reG);
	free(imG);
	free(reGt);
	free(imGt);
	free(reG_);
	free(imG_);
	free(reGt_);
	free(imGt_);

}


void ComplexSF::GenericfilterReal(double *realsrc, double *imagsrc, 
				double *realdestUp, double *imagdestUp, 
				double *realcoeffs, double *imagcoeffs,
				const int boarder, const int degree,
				const int central, const int order, 
				const int m, const int n)
{

	int mn = m*n;
	double *reG = (double*) malloc(sizeof(double)*mn);
	double *imG = (double*) malloc(sizeof(double)*mn);
	double *reG_ = (double*) malloc(sizeof(double)*mn);
	double *imG_ = (double*) malloc(sizeof(double)*mn);
	double *reGt = (double*) malloc(sizeof(double)*mn);
	double *imGt = (double*) malloc(sizeof(double)*mn);
	double *reGt_ = (double*) malloc(sizeof(double)*mn);
	double *imGt_ = (double*) malloc(sizeof(double)*mn);


	memcpy(reGt,realsrc,sizeof(double)*mn);
	if (imagsrc != 0)
		memcpy(imGt,imagsrc,sizeof(double)*mn);
	else
	{
		for (int i = 0;i < mn;i++) imGt[i] = 0.0;
	}		

	for (int k = 0;k < degree;k++)
	{
		if (k > 0)
		{
			if (k%2 == 0)	
				partialDeriv(reGt,imGt,reGt_,imGt_,boarder,1,-central,order,m,n);
			else
				partialDeriv(reGt,imGt,reGt_,imGt_,boarder,1,central,order,m,n);
			swap(reGt,reGt_);
			swap(imGt,imGt_);
		}

		memcpy(reG,reGt,sizeof(double)*mn);
		memcpy(imG,imGt,sizeof(double)*mn);
		double *redest;
		double *imdest;


		for (int j = 0;j <= k;j++)
		{
			if (j > 0)
			{
				if ((k+j)%2 == 0)
					partialDeriv(reG,imG,reG_,imG_,boarder,-1,-central,order,m,n);
				else
					partialDeriv(reG,imG,reG_,imG_,boarder,-1,central,order,m,n);
				swap(reG,reG_);
				swap(imG,imG_);
			}

			if (imagcoeffs == 0)
			{
		
				double realcoeff = realcoeffs[k+j*degree];
	
				redest = realdestUp + mn*(k-j);
				imdest = imagdestUp + mn*(k-j);

				for (int i = 0;i < mn; i++)
				{
					redest[i] += realcoeff*reG[i];
					imdest[i] += realcoeff*imG[i];
				}

			}
			else
			{
		
				double realcoeff = realcoeffs[k*degree+j];
				double imagcoeff = imagcoeffs[k*degree+j];

				redest = realdestUp + mn*(k-j);
				imdest = imagdestUp + mn*(k-j);

				for (int i = 0;i < mn; i++)
				{
					redest[i] += realcoeff*reG[i] - imagcoeff*imG[i];
					imdest[i] += realcoeff*imG[i] + imagcoeff*reG[i];
				}

			}

		}
	}

	free(reG);
	free(imG);
	free(reGt);
	free(imGt);
	free(reG_);
	free(imG_);
	free(reGt_);
	free(imGt_);

}





void ComplexSF::Genericfilter1(double *realsrc, double *imagsrc, 
			        double *realdest, double *imagdest, 
				double *realcoeffs, double *imagcoeffs,
				const int sign,
				const int boarder, const int degree,
				const int central, const int order, 
				const int m, const int n)
{

	int mn = m*n;
	double *reG = (double*) malloc(sizeof(double)*mn);
	double *imG = (double*) malloc(sizeof(double)*mn);
	double *reG_ = (double*) malloc(sizeof(double)*mn);
	double *imG_ = (double*) malloc(sizeof(double)*mn);


	memcpy(reG,realsrc,sizeof(double)*mn);
	if (imagsrc != 0)
		memcpy(imG,imagsrc,sizeof(double)*mn);
	else
	{
		for (int i = 0;i < mn;i++) imG[i] = 0.0;
	}		

	double *redest;
	double *imdest;

	for (int j = 0;j < degree;j++)
	{
		if (j > 0)
		{
			if (j%2 == 0)
				partialDeriv(reG,imG,reG_,imG_,boarder,sign,-central,order,m,n);
			else
				partialDeriv(reG,imG,reG_,imG_,boarder,sign,central,order,m,n);
			swap(reG,reG_);
			swap(imG,imG_);
		}

		if (imagcoeffs == 0)
		{
	
			double realcoeff = realcoeffs[j];

			redest = realdest + mn*j;
			imdest = imagdest + mn*j;

			for (int i = 0;i < mn; i++)
			{
				redest[i] += realcoeff*reG[i];
				imdest[i] += realcoeff*imG[i];
			}

		}
		else
		{
	
			double realcoeff = realcoeffs[j];
			double imagcoeff = imagcoeffs[j];

			redest = realdest + mn*j;
			imdest = imagdest + mn*j;

			for (int i = 0;i < mn; i++)
			{
				redest[i] += realcoeff*reG[i] - imagcoeff*imG[i];
				imdest[i] += realcoeff*imG[i] + imagcoeff*reG[i];
			}

		}

	}


	free(reG);
	free(imG);
	free(reG_);
	free(imG_);

}




