%
%  [destRe destIm] = GaborFilt(img,freq,gwid,[parameters])
% 
%  computes the steerable Gabor Filter.
%
%  Parameters:
%      img - image to be processed
%      freq - width of edge
%      gwid - width of Gaussian
%      parameters - structure with parameters for finite differences.
%         .order - first or second order (1 or 2)
%         .degree - expansion degree (integer >=1)
%         .type - central (0) or forward/backward (1) differences
%
%  returns reponses with positive rotation behavior in destRe
%  and negative in destIm.
%

function [destRe destIm] = GaborFilt(img,freq,gwid,parameters)

order = 1;
typ = 0;
A = 5;

if nargin == 4,
    if isfield(parameters,'order'),
        order = parameters.order;
    end;
    if isfield(parameters,'degree'),
        A = parameters.degree;
    end;
    if isfield(parameters,'type'),
        typ = parameters.type;
    end;
end


g = gaussBasisFilt(size(img),gwid);
g = ifft2(g .* fft2(img));
%g = img;

p=freq/4;


K = 0:(A-1);
alpha = ((-p).^K./factorial(K))' * (p.^K./factorial(K));
[destRe destIm] = ComplexFilter(g,alpha,order,typ,0);
