
#include "mex.h"
#include "matrix.h"

#include "ComplexSF.cpp"

void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[] )
{
 	
    if(nrhs> 5 || nrhs < 1) {
	printf("\nUsage: Df = Zderivative(f ,sign=1,boarder=0,order=1, type = 0)\n\n",nrhs);
	printf(" Computes complex derivative 4*df/dz or 4*df/dzbar, respectively. Input image values outside\n");
	printf(" the bounds of the array are computed by implicitly assuming the input array is periodic. \n\n");
	printf(" Parameters:\n");
	printf("   f - 2D input image of type double \n");
	printf("   sign - for sign=1 computes d/dz, sign=-1 computes d/dzbar\n" );
	printf("   boarder - defines width of the boundary where the derivative is NOT computed\n" );
	printf("   order - approximation order, possible values 1 or 2\n" );
	printf("   type - possible values -1,0,1 for backward, central, forward differences\n\n");
	printf(" Return value Df contains the result, same size as f.\n\n");
    return;
	} else if(nlhs>1) {
	printf("Too many output arguments\n");
    return;
	}

    int pcnt = 0;
    const mxArray *Img;
    Img = prhs[pcnt++];       
    const int numdim = mxGetNumberOfDimensions(Img);
    const int *dims = mxGetDimensions(Img);
    const int d1 = dims[0];
    const int d2 = dims[1];

    double *realimg = mxGetPr(Img);
    double *imagimg = mxGetPi(Img);

    int sign = 1;
    int boarder = 0;
    int order = 1;
    int central = 0;

    if (nrhs > 1)
	{
    	double *f_sign = mxGetPr(prhs[pcnt++]);
    	sign = (*f_sign > 0)? 1 : -1;
	}
    if (nrhs > 2)
	{
	double *f_boarder = mxGetPr(prhs[pcnt++]);
	boarder = (int) *f_boarder; 
	}
    if (nrhs > 3)
	{
	double *f_order = mxGetPr(prhs[pcnt++]);
	if (*f_order > 1.5)
		order = 2;
	}
    if (nrhs > 4)
	{
	double *f_central = mxGetPr(prhs[pcnt++]);
	central = (int)(*f_central);
	}

    plhs[0] = mxCreateDoubleMatrix(d1,d2,mxCOMPLEX);
    double *resreal = mxGetPr(plhs[0]);
    double *resimag = mxGetPi(plhs[0]);

    ComplexSF::partialDeriv(realimg,imagimg,resreal,resimag,boarder,sign,central,order,d1,d2);

}

