%
%   response = steerFilter(osamp, steerfun, destRe, destIm)
%
%   steers the filterresponses given by destRe and destIm 
%   according the given steeringfunction with angle resolution 
%   of size(destRe,3)*osamp.
% 
%   Parameters:
%       osamp - oversampling factor 
%       steerfun - steeringfunction that takes a 3D array where first 
%          dimension contains the orientation specific filter 
%          responses. 2nd and 3rd dimension the image information.  
%       destRe, destIm - contains the stterable filter responses. 
%
%   returns reponses in complex notation. Phase corresponds to orientation
%   and magnitude to strength.
%
%   Example:
%      [destRe destIm] = GaborFilt(img,freq,width,filtparam); % apply filter
%      maximag = @(x) max(imag(x));  % define steeringfunction
%      response = steerFilter(5,maximag,destRe,destIm); % steer filter
%


function response = steerFilter(osamp, steerfun, destRe, destIm)
fun = fcnchk(steerfun);
if nargin == 4,    

    A = size(destRe,3);    
    filtresponse = fft(destRe,osamp*A,3)+conj(fft(conj(destIm),osamp*A,3));
    [magnitude direction] = fun(permute(filtresponse,[3 1 2]));
    response = magnitude .* exp(i*2*pi*direction/(osamp*A));
    
else if nargin == 3,        

    A = size(destRe,3);   
    filtresponse = fft(destRe,osamp*A,3);
    [magnitude direction] = fun(permute(filtresponse,[3 1 2]));
    response = magnitude .* exp(i*2*pi*direction/(osamp*A));
    else,
    
        display('Wrong number of arguments!');
        return
    end;
end;
    
response = permute(response,[2 3 1]);

