

display('-----------------------------------');

%% loading example image
img = double(imread('testimg.pgm'))/255;
template = double(imread('template.pgm'))/255;

%% generating unitpulse
n = 64;
unitpulse = zeros(n,n);
unitpulse(ceil(n/2),ceil(n/2)) = 1;

%% steering functions
maximag = @(x) max(imag(x));
maxreal = @(x) max(real(x));

%% setting filter parameters
filtparam.degree = 12;
filtparam.order = 1;
filtparam.type = 0;

oversampling = 4;


img = img+randn(size(img))*0.5;




%%%%%%%%%%%%%%%%%%%%%% testing decomposition %%%%%%%%%%%%%%%%%%%

display('decomposition of template');

%% compute decomposition
width = 6;
template = imresize(template,[64 64],'bilinear');
filtermodel = decomposeKernel(template,width,filtparam);

display('applying filter');

%% apply filter
destRe = applyFilter(img,filtermodel);
destRe(:,:,1) = 0;

display('steering and showing results');

%% steer by maximum of real part and show magnitude of the result.
figure(2);
response = steerFilter(oversampling,maxreal,destRe);
imagesc(abs(response).^2);


display('----------------------------------- press a key');
pause



%%%%%%%%%%%%%%%%%%%%%%% testing Gabor filter %%%%%%%%%%%%%%%%%%%%%%
filtparam.degree = 7;
freq = 8;
width = 3;

display('show impulse reponse of Gabor filter');

%% compute and show impulse response
[destRe destIm] = GaborFilt(unitpulse,freq,width,filtparam);
filterkernel = sum(destRe,3) + sum(destIm,3);
figure(1);
subplot(1,2,1);
imagesc(real(filterkernel));
subplot(1,2,2);
imagesc(imag(filterkernel));

display('applying Gabor Filter');

%% apply on testimage
[destRe destIm] = GaborFilt(img,freq,width,filtparam);


display('steering and showing results');

%% steer by maximum of imaginary part and show magnitude of the result.
figure(2);
response = steerFilter(oversampling,maxreal,destRe,destIm);
imagesc(abs(response));

display('----------------------------------- press a key');
pause



%%%%%%%%%%%%%%%%%%%%%%% testing Pin filter %%%%%%%%%%%%%%%%%%%%%%

freq = 4;
width = 3;

display('show impulse reponse of Pin filter');

%% compute and show impulse response
destRe = PinFilt(unitpulse,freq,width,filtparam);
filterkernel = sum(destRe,3);
figure(1);
subplot(1,2,1);
imagesc(real(filterkernel));
subplot(1,2,2);
imagesc(imag(filterkernel));


display('applying Pin filter');

%% apply on testimage
destRe = PinFilt(img,freq,width,filtparam);

display('steering and showing results');

%% steer by maximum of real part and show magnitude of the result.
figure(2);
response = steerFilter(oversampling,maxreal,destRe);
imagesc(abs(response));

display('----------------------------------- press a key ');
pause


%%%%%%%%%%%%%%%%%%%%%%% testing Edge/Ridge filter %%%%%%%%%%%%%%%%%%%%%%


freq = 5;
width = 3;

display('show impulse reponse of Edge/Ridge filter');

%% compute and show impulse response
[destRe destIm] = EdgeRidgeFilt(unitpulse,freq,width,filtparam);
filterkernel = sum(destRe,3) + sum(destIm,3);
figure(1);
subplot(1,2,1);
imagesc(real(filterkernel));
subplot(1,2,2);
imagesc(imag(filterkernel));

display('applying Edge/Ridge Filter');

%% apply on testimage
[destRe destIm] = EdgeRidgeFilt(img,freq,width,filtparam);

display('steering and showing results');

%% steer by maximum of imaginary part and show magnitude of the result.
figure(2);
response = steerFilter(oversampling,maximag,destRe,destIm);
imagesc(abs(response));



